package test;

import java.io.FilterOutputStream;
import java.io.FilterWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.Enumeration;
import java.util.HashMap;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.WriteListener;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

@WebFilter(filterName="LogFilter",urlPatterns={"*"})
public class LogFilter implements Filter {
	public static class LogOutputStream extends FilterOutputStream {
		public LogOutputStream(OutputStream real) {
			super(real);
		}
		@Override
		public void write(byte[] b) throws IOException {
			System.out.print(new String(b));
			super.write(b);
		}
		@Override
		public void write(byte[] b, int off, int len) throws IOException {
			System.out.print(new String(b, off, len));
			super.write(b, off, len);
		}
		@Override
		public void write(int b) throws IOException {
			System.out.print((char)b);
			super.write(b);
		}
	}
	public static class LogWriter extends FilterWriter {
		protected LogWriter(Writer real) {
			super(real);
		}
		@Override
		public void write(char[] cbuf, int off, int len) throws IOException {
			System.out.print(new String(cbuf, off, len));
			super.write(cbuf, off, len);
		}
		@Override
		public void write(int c) throws IOException {
			System.out.print((char)c);
			super.write(c);
		}
		@Override
		public void write(String str, int off, int len) throws IOException {
			System.out.println(str.substring(off, off + len));
			super.write(str, off, len);
		}
	}
	public static class LogServletOutputStream extends ServletOutputStream {
		private OutputStream real;
		public LogServletOutputStream(OutputStream real) {
			this.real = real;
		}
		@Override
		public boolean isReady() {
			return true;
		}
		@Override
		public void setWriteListener(WriteListener wrtlist) {
		}
		@Override
		public void write(int b) throws IOException {
			real.write(b);
		}
	}
	public static class LogHttpServletResponse extends HttpServletResponseWrapper {
		private HttpServletResponse real;
		public LogHttpServletResponse(HttpServletResponse real) {
			super(real);
			this.real = real;
		}
        @Override
		public ServletOutputStream getOutputStream() throws IOException {
			return new LogServletOutputStream(new LogOutputStream(real.getOutputStream()));
		}
        @Override
		public PrintWriter getWriter() throws IOException {
			return new PrintWriter(new LogWriter(real.getWriter()));
		}
	}
    @Override
    public void init(FilterConfig conf) throws ServletException {
    }
    @Override
    public void doFilter(ServletRequest req, ServletResponse resp, FilterChain chain) throws IOException, ServletException {
    	HttpServletRequest httpreq = (HttpServletRequest)req;
    	HttpServletResponse httpresp = (HttpServletResponse)resp;
    	System.out.println(httpreq.getMethod() + " " + httpreq.getRequestURI() + " " + httpreq.getProtocol());
    	Enumeration<String> ehdr = httpreq.getHeaderNames();
    	while(ehdr.hasMoreElements()) {
    		String hdr = ehdr.nextElement();
    		System.out.println(hdr + ": " + httpreq.getHeader(hdr));
    	}
    	System.out.println("Parameters = " + new HashMap<String, String[]>(httpreq.getParameterMap()));
    	chain.doFilter(httpreq, new LogHttpServletResponse(httpresp));
    	for(String hdr : httpresp.getHeaderNames()) {
    		System.out.println(hdr + ": " + httpresp.getHeader(hdr));
    	}
    }
    @Override
    public void destroy() {
    }
}
